/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.webidor.resources;

import com.sun.jersey.api.container.httpserver.HttpServerFactory;
import com.sun.jersey.api.core.PackagesResourceConfig;
import com.sun.jersey.api.core.ResourceConfig;
import com.sun.jersey.spi.resource.Singleton;
import com.sun.net.httpserver.HttpServer;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.ServerSocket;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.UUID;
import javax.ws.rs.GET;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
@Path("/api")
@Singleton
public final class Quoridor {
    private final File path;
    private Games games;
    private final Map<UUID,String> loggedIn;

    public Quoridor() {
        final String prop = System.getProperty("quoridor.dir"); // NOI18N
        if (prop == null) {
            throw new IllegalStateException("quoridor.dir property must be specified"); // NOI18N
        }
        path = new File(prop);
        path.mkdirs();
        loggedIn = new HashMap<UUID, String>();
    }

    @Path("games")
    public Games getGames() {
        if (games == null) {
            games = new Games(new File(path, "games"), this); // NOI18N
        }
        return games;
    }

    @Path("login")
    @PUT
    @Produces({ MediaType.TEXT_PLAIN, MediaType.APPLICATION_JSON, MediaType.TEXT_XML })
    public String login(
        @QueryParam("name") String name,
        @QueryParam("password") String password
    ) {
        File f = new File(path, "passwd"); // NOI18Nt
        Properties p = new Properties();
        try {
            p.load(new FileInputStream(f));
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        if (name != null && password.equals(p.getProperty(name))) {
            UUID uuid = UUID.randomUUID();
            loggedIn.put(uuid, name);
            return uuid.toString();
        } else {
            return null;
        }
        
    }

    @Path("login")
    @GET
    @Produces({ MediaType.TEXT_PLAIN, MediaType.APPLICATION_JSON, MediaType.TEXT_XML })
    public String isLoggedIn(
        @QueryParam("id") String id
    ) {
        return id == null ? null : loggedIn.get(UUID.fromString(id));
    }

    //
    // start the server
    //

    public static void main(String[] args) throws IOException {
        HttpServer s = start(9998);
        System.out.println(
            "Quoridor started at port 9998\n" + "Hit enter to stop it..."
        );
        System.in.read();
        s.stop(0);
        System.exit(0);
    }

    public static HttpServer start(int port) throws IOException {
        if (port == -1) {
            ServerSocket ss = new ServerSocket(0);
            port =ss.getLocalPort();
            ss.close();
        }
        final String baseUri = "http://localhost:" + port + "/";

        File home = new File(System.getProperty("user.home"));
        File quoridor = new File(home, ".quoridor");

        System.setProperty("quoridor.dir", quoridor.getPath());

        ResourceConfig rc = new PackagesResourceConfig("cz.xelfi.quoridor.webidor");
        HttpServer server = HttpServerFactory.create(baseUri, rc);
        server.start();
        return server;
    }

}
